<?php
/**
 * Diese Datei ist Teil von „Lernstrategien“ 
 * (Dateiname: „mod_splernstrategien“), einer Plugin-Software 
 * für das moodle LMS.
 *
 * Das moodle LMS ist lizenziert mit der GNU General Public 
 * License Version 3. Aktuelle Lizenzhinweise zu moodle 
 * finden Sie unter: https://moodledev.io/general/license.
 *
 * Das Plugin ist lizenziert mit der GNU General Public 
 * License Version 3. Anzugebender Urheber des Plugins ist
 * die „metromorph softworks GmbH im Auftrag von ORCA.nrw“.
 * Die Lizenzierung umfasst nicht die Inhalte, die auf der
 * Anwenderoberfläche des Plugins angezeigt werden; diese sind 
 * ihrerseits CC BY-SA 4.0 lizenziert.
 *
 * Sie sollten eine Kopie der GNU-Lizenz zusammen mit diesem 
 * Programm erhalten haben. Sie finden sie auch unter 
 * https://www.gnu.org/licenses. Die CC-Lizenz finden Sie
 * unter https://creativecommons.org/licenses/by-sa/4.0/deed.de.
 */

namespace mod_splernstrategien;

defined('MOODLE_INTERNAL') || die();


/**
 * Lernstrategien user representation
 * 
 * @package   mod_splernstrategien
 * @license   https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */
class user
{
    /**
     * The new users defaults.
     */
    public const DEFAULTS = [
        'moodle_id' => NULL,
        'course' => NULL,
        'user_id' => NULL,
        'timecreated' => NULL,
        'epa_finished' => 0,
        'epb_finished' => 0,
        'wp_finished' => 0,
        'delay' => 0,
        'timestamp' => 0,
        'laststate' => 1,
        'nextstate' => 0,
    ];

    /** @var array user answers. */
    public $answers = [];

    /** @var int record id in answers table. */
    protected $answersid = NULL;

    /**
     * Constructor is internal, use from_ or init static methods to get a LS user.
     * 
     * @param array $array contains all user properties as assoc array.
     */
    private function __construct($array)
    {
        // Add every array field as property (stdClass-ish).
        foreach ($array as $k => $v) {
            $this->$k = $v;
        }
    }

    /**
     * Init new LS user if not existing, do nothing otherwise.
     *
     * @param object $mdluser the moodle user object.
     * @param int $course_id course id.
     * @param int $cm_id module id.
     */
    public static function init($mdluser, $course_id, $cm_id)
    {
        global $DB;
        $table = "splernstrategien_users"; //TODO read from plugin-config

        $user = array_merge(self::DEFAULTS, [
            'moodle_id' => $cm_id,
            'course' => $course_id,
            'user_id' => $mdluser->id,
            'timecreated' =>  date_timestamp_get(date_create())
        ]);
        // \todo\test This should throw, when user already exists, should never pass silent when no
        // user is initialized (created or overwritten).
        $user['id'] = $DB->insert_record($table, $user);

        return new static($user);
    }

    /**
     * Get LS user from moodle user and course module ids.
     *
     * @param object $mdluser the moodle user object.
     * @param int $course_id course id.
     * @param int $cm_id module id.
     * @return object Returns the user object if found, false otherwise.
     */
    public static function from_moodleuser($mdluser, $course_id, $cm_id)
    {
        global $DB;

        $record = $DB->get_record(
            "splernstrategien_users",
            ['user_id' => $mdluser->id, 'course' => $course_id, 'moodle_id' => $cm_id],
            $fields = '*',
            $strictness = IGNORE_MISSING
        );

        if ($record === false) {
            return false;
        }

        $obj = new static($record);

        // \todo\legacy Is this needed? How can laststate become NULL?
        if (is_null($obj->laststate)) {
            $obj->laststate = 1;
        }

        // Add the users answers to the object.
        $answers = $DB->get_record(
            "splernstrategien_answers",
            ['user_id' => $obj->user_id, 'course' => $obj->course, 'moodle_id' => $obj->moodle_id],
            $fields = '*',
            $strictness = IGNORE_MISSING
        );

        if ($answers !== false) {
            $obj->answersid = $answers->id;
            $obj->answers = json_decode($answers->data, true);
        }

        return $obj;
    }

    /**
     * Save user data to DB.
     */
    public function save()
    {
        global $DB;
        $table = "splernstrategien_answers";

        $answersdata = json_encode($this->answers);
        if (is_null($this->answersid)) {
            $this->answersid = $DB->insert_record($table, [
                'moodle_id' => $this->moodle_id,
                'user_id' => $this->user_id,
                'course' => $this->course,
                'data' => $answersdata
            ]);
        } else {
            $DB->update_record($table, ['id' => $this->answersid, 'data' => $answersdata]);
        }

        $userrecord = ['id' => $this->id];
        foreach (array_keys(self::DEFAULTS) as $k) {
            $userrecord[$k] = $this->$k;
        }
        $DB->update_record("splernstrategien_users", $userrecord);
    }

    /**
     * Update current answers with new ones or replace old ones.
     *
     * @param array $answers to merge into the current ones.
     * @return array Returns the updated answers.
     */
    public function merge_answers($answers)
    {
        $this->answers = array_merge($this->answers, $answers);
        return $this->answers;
    }

    /**
     * Check if user finished all "Stränge".
     */
    public function finished_all()
    {
        return $this->epa_finished && $this->epb_finished && $this->wp_finished;
    }

    /**
     * Return finished Stränge as ordered array.
     */
    public function finished_array()
    {
        return [$this->epa_finished, $this->epb_finished, $this->wp_finished];
    }

    /**
     * Set finished Strang by number.
     * 
     * @param int $n Nuber of Strang.
     */
    public function set_finished($n)
    {
        switch ($n) {
            case 1:
                $this->epa_finished = 1;
                break;
            case 2:
                $this->epb_finished = 1;
                break;
            case 3:
                $this->wp_finished = 1;
                break;
        }
    }

    /**
     * The calculate delay left. Negative values mean there is none (anymore).
     */
    function delay_left()
    {
        $now = date_timestamp_get(date_create());
        return $this->timestamp - $now + $this->delay;
    }
}
